from abc import ABCMeta, abstractmethod
# Interfaces werden ähnlich wie Klassen aufgeschrieben    
class DNASequenceADT(metaclass=ABCMeta):
  @abstractmethod
  def print(self): pass

  @abstractmethod
  def concat(self, sequenz: "DNASequenceADT"): pass

  # Man listet die gewünschten Methoden auf.
  # Man darf keine Attribute angeben und die Methoden
  # dürfen nicht implementiert werden.


class DNASequence_1:
  __metaclass__= DNASequenceADT

  def __init__(self, base: "DNASequenceADT"):
    # Erzeuge eine DNA-Sequenz mit nur einer Base
    self.bases= base

  # Methoden
  def __len__(self):
    return self.bases.__len__()

  def __iter__(self):
    return self.bases.__iter__()
  
  def print(self):
    for elem in self.bases:
      print (elem, end= "")
    print ()  


  def concat (self, seq: "DNASequenceADT"):
    # Die Basen aus seq sollen an das Ende von bases
    # angefügt werden.

    # Neuen Array erzeugen:
    new_array= [""] * (self.bases.__len__() + seq.__len__())

    # Kopiere eigenen Basen an den Anfang:
    i= 0
    for elem in self.bases:
      new_array[i]= self.bases[i]
      i= i+1

    # Kopiere die anderen Basen dahinter:
    for elem in seq:
      new_array[(i-1) + self.bases.__len__()]= elem
      i= i+1

    # Der neue Basenarray ist nun der aktuelle
    self.bases= new_array

if __name__ == "__main__":
  print ("Interface wird mit Array realisiert:")
  a= DNASequence_1('A')
  b= DNASequence_1('C')
  
  a.print()
  b.print()
  
  a.concat(b)
  a.print() 

# Listen -- in allen Fällen wird die Klasse Cell benötigt.
class Cell:
  def __init__(self):
    self.base= ""
    self.next= None

#------------------------------------ Liste einfach --------------------------
class DNASequence_2:
  __metaclass__= DNASequenceADT

  # Konstruktor
  def __init__(self, bas: "DNASequenceADT"):
    # Erzeuge eine DNA-Sequenz mit nur einer Base
    self.start     = Cell()
    self.start.base= bas
    self.start.next= None

  # Methoden
  def print(self):
    cursor= self.start
    while cursor != None:
      print(cursor.base,end= "")
      cursor= cursor.next
    print()  


  def concat (self, seq: "DNASequenceADT"):
    # Die Basen in seq sollen an das Ende von bases
    # angefügt werden.

    # Finde das Ende:
    cursor= self.start
    while cursor.next != None:
      cursor= cursor.next

    # Hänge seq an:
    cursor.next= seq.start


if __name__ == "__main__":
  print ("Interface wird mit Liste realisiert:")
  a= DNASequence_2('A')
  b= DNASequence_2('C')
  a.print()
  b.print()

  a.concat(b)
  a.print() 

#------------------------------------ Liste einfach, mit Ende ----------------
class DNASequence_3:
  __metaclass__= DNASequenceADT

  # Konstruktor
  def __init__(self, bas: "DNASequenceADT"):
    # Erzeuge eine DNA-Sequenz mit nur einer Base
    self.start     = Cell()
    self.start.base= bas
    self.start.next= None
    self.end= self.start

  # Methoden
  def print(self):
    cursor= self.start
    while cursor != None:
      print(cursor.base,end= "")
      cursor= cursor.next
    print()  

  def concat (self, seq: "DNASequenceADT"):
    # Die Basen in seq sollen an das Ende von bases
    # angefügt werden.
    self.end.next= seq.start
    self.end     = seq.end

if __name__ == "__main__":
  print ("Interface wird mit Liste (inkl. Endemarkierung) realisiert:")
  a= DNASequence_3('A')
  b= DNASequence_3('C')
  a.print()
  b.print()

  a.concat(b)
  a.print() 

#------------------------------------ Liste doppelt, mit Ende -----------------
class Cell:
  def __init__(self):
    self.base= ""
    self.next= None
    self.prev= None


class DNASequence_4:
  __metaclass__= DNASequenceADT

  # Konstruktor
  def __init__(self, bas: "DNASequenceADT"):
    # Erzeuge eine DNA-Sequenz mit nur einer Base
    self.start     = Cell()
    self.start.base= bas
    self.start.next= None
    self.start.prev= None
    self.end       = self.start

  # Methoden
  def print(self):
    cursor= self.start
    while cursor != None:
      print(cursor.base,end= "")
      cursor= cursor.next
    print()  

  def concat (self, seq: "DNASequenceADT"):
    # Die Basen in seq sollen an das Ende von bases
    # angefügt werden.
    self.end.next = seq.start
    seq.start.prev= self.end
    self.end      = seq.end

if __name__ == "__main__":
  print ("Interface wird mit doppelt verketteter Liste realisiert:")
  a= DNASequence_4('A')
  b= DNASequence_4('C')
  a.print()
  b.print()

  a.concat (b)
  a.print() 
  
    
#------------------------------------ Set -------------------------------------
class Menge(metaclass= ABCMeta):
  @abstractmethod
  def putIn(self, zeichenkette: "str"): pass

  @abstractmethod
  def isIn(self, zeichenkette: "str") -> "bool": return

#---------------------------- Sortierte Liste zur Speicherung der Zeichenketten
class Set_as_array:
  __metaclass__= Menge

  def __init__(self):
    self.zeichenketten= [""]

  # Methoden
  def print(self):
    # tbd.
    pass

  def putIn(self, zeichenkette: "str"): 
    new_array= [""] * (self.zeichenketten.__len__() + 1)
    # Kopiere alle Strings kleiner als zeichenkette nach new_array
    # tbd...
    # Kopiere zeichenkette nach new_array
    # tbd...
    # Kopiere alle Strings größer s nach new_array
    self.zeichenketten= new_array

  def isIn(self, zeichenkette: "str") -> "bool": 
    # Suche nach zeichenkette in self.zeichenketten mittels binärer Suche
    ergebnis= False
    return ergebnis

if __name__ == "__main__":
  print ("Interface wird mit sortierter Liste zur Speicherung realisiert:")
  a= Set_as_array()
  a.putIn ('A')
  b= Set_as_array()
  b.putIn ('C')
  #a.print()
  #b.print()

  #a.concat (b)
  #a.print() 
  

    
#--------------------------- Unsortiere Liste zur Speicherung der Zeichenketten
# tbd.
