"""
$Id: phylogenetischerBaum_Typen.py 1382 2015-12-18 22:42:02Z humbert $
$URL: svn+ssh://l119v.studs.math.uni-wuppertal.de/home/mueller/Kooperation/CDG/repos/Materialsammlung/trunk/oberstufe/skriptum/python_quellen/phylogenetischerBaum.py $

 Beispiel: Vorhaben Q1-3
 Python3-Programm zum phylogenetischen Baum 
 -- Methoden mit Typen für Anfragen und formale Parameter
  ...
  
"""

class Spezies:
  def __init__(self, name: 'str', genombe: 'str'):
    self.speziesName= name
    self.genom      = genombe


class Node:
  def __init__(self):
    # Die Spezies, die der Knoten speichert
    self.spezies   = None  # Spezies(..., ...)
    self.leftChild = None  # Node
    self.rightChild= None  # Node
    """
    # andere Variante: beliebig viele Kinder
    # Eine ganze Brut von Kindern...
    self.children= [] # Liste aus Elementen der Klasse Node()
    """

  def __init__(self, spez: 'Spezies'):
    self.spezies   = spez  # Spezies(..., ...) 
    self.leftChild = None  # Node
    self.rightChild= None  # Node
    

class PhylogeneticTree:   
  def __init__(self):
    self.root= None    # Wurzel - Klasse Node
    
  # Konstruktor, der einen Baum aufbaut
  # mit bereits konstruierten linken und rechten
  # Teilbäumen. Die Wurzel wird neu erzeugt.
  def __init__(self, 
      spezies: 'Spezies', 
      left: 'PhylogeneticTree', right: 'PhylogeneticTree'):
    self.root= Node(spezies)
    if left != None: 
      self.root.leftChild= left.root
    if right != None:
      self.root.rightChild= right.root

  
  # Methoden
  def isEmpty(self):
    return self.root == None

  def addTreeAsLeftChild(self, 
      where: 'PhylogeneticTree', 
      what: 'PhylogeneticTree'):
    where.leftChild= what.root
  
  def removeLeftSubtree(self, where: 'PhylogeneticTree'):
    where.leftChild= None

  def countNodesInSubtree(self, node: 'Node'):
    # Rekursive Post-Order-Traversierung
    if node == None:
      return 0
    else: 
      return self.countNodesInSubtree(node.leftChild) + \
             self.countNodesInSubtree(node.rightChild) + \
             1
           
  def countNodeInTree(self):
    return self.countNodesInSubtree(self.root)

  def printAllSpeziesPreOrder(self, node: 'Node'):
    if node != None:
      print(node.spezies.speziesName)
      self.printAllSpeziesPreOrder(node.leftChild)
      self.printAllSpeziesPreOrder(node.rightChild)

  def printAllSpeziesPostOrder(self, node: 'Node'):
    if node != None:
      self.printAllSpeziesPostOrder(node.leftChild)
      self.printAllSpeziesPostOrder(node.rightChild)
      print(node.spezies.speziesName)

  def printAllSpeziesInOrder(self, node: 'Node'):
    if node != None:
      self.printAllSpeziesInOrder(node.leftChild)
      print(node.spezies.speziesName)
      self.printAllSpeziesInOrder(node.rightChild)


if __name__ == "__main__":
  start = Spezies("first cell", "CCCT")
  human = Spezies("human",      "ACGT")
  monkey= Spezies("monkey",     "ACCT")
  fish  = Spezies("Hommingberger Gepardenforelle", "CTCT")

  myTree= PhylogeneticTree(start,               \
           PhylogeneticTree(monkey, None,       \
           PhylogeneticTree(human, None, None)  \
           ),                                   \
           PhylogeneticTree(fish, None, None)   \
          )
    
  print("Anzahl der Knoten:",myTree.countNodeInTree())
  
  myTree.printAllSpeziesInOrder(myTree.root)
  myTree.printAllSpeziesPreOrder(myTree.root)
  myTree.printAllSpeziesPostOrder(myTree.root)


