# Sebastian Raschka 2014 -- translated to German: Ludger Humbert 2016
# Gibt Daten einer SQLite Datenbank aus.

# z.B.
#
"""
Gesamtzeilenzahl: 1

Spalteninfo:
ID, Name, Typ, NichtNull, Voreinstellungswert, Primärschlüssel
(0, 'id', 'TEXT', 0, None, 1)
(1, 'date', '', 0, None, 0)
(2, 'time', '', 0, None, 0)
(3, 'date_time', '', 0, None, 0)

Einträge pro Spalte:
date: 1
date_time: 1
id: 1
time: 1
[('some_id1', '2016-05-24', '14:25:09', '2016-05-24 14:25:09')]
"""

import sqlite3

def connect(sqlite_datei):
    """ Stelle eine Verbindung zu einer SQLite-Datenbankdatei her """
    verbindungDB= sqlite3.connect(sqlite_datei)
    pos_zeig= verbindungDB.cursor()
    return verbindungDB, pos_zeig


def close(verbindungDB):
    """ Bestätige Änderungen und schließe die Verbindung zur Datenbank """
    #verbindungDB.commit()
    verbindungDB.close()


def gesamtzahl_zeilen(pos_zeig, tabelle_name, ausgeben= False):
    """ Liefert die Gesamtzahl der Zeilen der Datenbank """
    pos_zeig.execute('SELECT COUNT(*) FROM {}'\
        .format(tabelle_name))
    zaehler= pos_zeig.fetchall()
    if ausgeben:
        print('\nGesamtzeilenzahl: {}'.format(zaehler[0][0]))
    return zaehler[0][0]


def tabelle_spalte_info(pos_zeig, tabelle_name, ausgeben= False):
    """ 
      Liefert eine Liste von Tupeln der Spaltendaten:
      (id, name, typ, notnull, default_value, primary_key)
    
    """
    pos_zeig.execute('PRAGMA TABLE_INFO({})'.format(tabelle_name))
    info= pos_zeig.fetchall()
    
    if ausgeben:
        print("\nSpalteninfo:\nID, Name, Typ, NichtNull, \
Voreinstellungswert, Primärschlüssel")
        for spalte in info:
            print(spalte)
    return info


def werte_in_spalten(pos_zeig, tabelle_name, ausgeben= True):
    """ Liefert ein >>Wörterbuch<< mit Spalten als Schlüssel und der Anzahl
        der nicht leeren Einträge als zugehörige Werte.
    """
    pos_zeig.execute('PRAGMA TABLE_INFO({})'.format(tabelle_name))
    info= pos_zeig.fetchall()
    spalte_dict= dict()
    for spalte in info:
        spalte_dict[spalte[1]]= 0
    for spalte in spalte_dict:
        pos_zeig.execute('SELECT ({0}) FROM {1} WHERE {0} IS NOT NULL'\
            .format(spalte, tabelle_name))
        # In my case this approach resulted in a better performance 
        # than using COUNT
        anzahl_zeilen= len(pos_zeig.fetchall())
        spalte_dict[spalte]= anzahl_zeilen
    if ausgeben:
        print("\nEinträge pro Spalte:")
        for i in spalte_dict.items():
            print('{}: {}'.format(i[0], i[1]))
    return spalte_dict


def gesamter_inhalt(pos_zeig, tabelle_name, ausgeben= False):
    pos_zeig.execute('SELECT * FROM {}'\
        .format(tabelle_name))
    if ausgeben:
        print (pos_zeig.fetchall())


def inhalt_n_zeilen(pos_zeig, tabelle_name, zeilenzahl, ausgeben= False):
    pos_zeig.execute('SELECT * FROM {} LIMIT {}'\
        .format(tabelle_name, zeilenzahl))
    if ausgeben:
        print (pos_zeig.fetchall())
        
        
def alles_ausgeben(tabellen):
    for tabelle_name in tabellen:
        gesamtzahl_zeilen(pos_zeig, tabelle_name, ausgeben= True)   
        tabelle_spalte_info(pos_zeig, tabelle_name, ausgeben= True)
        werte_in_spalten(pos_zeig, tabelle_name, ausgeben= True) 
        gesamter_inhalt(pos_zeig, tabelle_name, ausgeben= True)
#        inhalt_n_zeilen(pos_zeig, tabelle_name, 3, ausgeben= True)

if __name__ == '__main__':
    sqlite_datei= '/sdcard/db/lh_db.sqlite'
    verbindungDB, pos_zeig= connect(sqlite_datei)

    tabellen= ['lh_tabelle_1','lh_tabelle_2','lh_tabelle_3',"schaf","stall","liegt_in"]
    alles_ausgeben(tabellen)
   
    close(verbindungDB)

