import sqlite3

def erzeuge_spalten_index(db_dateiname, tabelle_name, spalten_name, index_name):
    '''
    Erzeugt einen Spaltenindex für eine SQLite Tabelle.
    
    Keyword arguments:
        db_dateiname(str): Pfad zu der .sqlite-Datenbankdatei.
        tabelle_name(str): Name der Tabelle in der SQLite-Datei.
        bedingung(str): Bedingung für die Abfrage der Tabelle der
                        SQLite-Datenbank.
        spalten_name(str): Name der Spalte, für die der Index erstellt wird.
    '''

    # Connecting to the database file
    verbindungDB= sqlite3.connect(db_dateiname)
    pos_zeig= verbindungDB.cursor()

    # Creating the index
    pos_zeig.execute('CREATE INDEX {} ON {} ({})'\
        .format(index_name, tabelle_name, spalten_name))

    # Save index and close the connection to the database
    verbindungDB.commit()
    verbindungDB.close()


def entferne_spalten_index(db_dateiname, index_name):
    '''
    Entfernt einen Spaltenindex einer SQLite Tabelle.
    
    Keyword arguments:
        db_dateiname(str): Pfad zu der .sqlite-Datenbankdatei.
        tabelle_name(str): Name der Tabelle in der SQLite-Datei.
        bedingung(str): Bedingung für die Abfrage der Tabelle der
                        SQLite-Datenbank.
        spalten_name(str): Name der Spalte, für die der Index entfernt wird.
    '''

    # Connecting to the database file
    verbindungDB= sqlite3.connect(db_dateiname)
    pos_zeig= verbindungDB.cursor()

    # Drops the index
    pos_zeig.execute('DROP INDEX {}'.format(index_name))

    # Save index and close the connection to the database
    verbindungDB.commit()
    verbindungDB.close()


def schreibe_nach_abfrage(db_dateiname, tabelle_name, bedingung, inhalt_spalte, 
    ausgabe_datei, fetchmany=False):
    '''
    Schreibt den Inhalt einer Spalte in eine Datei
    
    Keyword arguments:
        db_dateiname(str): Pfad zu der .sqlite-Datenbankdatei.
        tabelle_name(str): Name der Tabelle in der SQLite-Datei.
        bedingung(str): Bedingung für die Abfrage der Tabelle der
                        SQLite-Datenbank.
        inhalt_spalte(str): Name der Spalte, die den Inhalt für 
                            die Ausgabedatei enthält.
        ausgabe_datei(str): Pfad zur Ausgabedatei.

    '''

    # Connecting to the database file
    verbindungDB= sqlite3.connect(db_dateiname)
    pos_zeig= verbindungDB.cursor()

    # Querying the database and writing the output file


    # A) using .fetchmany(); recommended for larger databases
    if fetchmany:
        pos_zeig.execute('SELECT ({}) FROM {} WHERE {}'\
            .format(inhalt_spalte, tabelle_name, bedingung))
        with open(ausgabe_datei, 'w') as ausdatei:
            ergebnisse = pos_zeig.fetchmany(fetchmany)
            while ergebnisse:
                for row in ergebnisse:
                    ausdatei.write(row[0])
                ergebnisse = pos_zeig.fetchmany(fetchmany)

    # B) simple .execute() loop
    else:
        pos_zeig.execute('SELECT ({}) FROM {} WHERE {}'.format(inhalt_spalte, tabelle_name, bedingung))
        with open(ausgabe_datei, 'w') as ausdatei:
            for row in pos_zeig:
                ausdatei.write(row[0])

    # Closing the connection to the database
    verbindungDB.close()

if __name__ == '__main__':
    schreibe_nach_abfrage(
        db_dateiname='/sdcard/db/lh_db.sqlite',
        tabelle_name='lh_tabelle_2',
        bedingung='variable1=1 AND variable2<=5 AND variable3="Zinc_Plus"',
        inhalt_spalte='variable4',
        ausgabe_datei='/sdcard/db/lh_sqlite_out.txt'
    )
        


